import uuid

from flask_login import current_user
from flask import jsonify, send_from_directory, url_for
# from markdown_it.common.html_re import comment
from sqlalchemy.orm import sessionmaker
from datetime import datetime
from k2.k2obj import K2Obj
from sqlalchemy.testing.plugin.plugin_base import logging
from sqlalchemy.orm import aliased
from sqlalchemy import or_, and_, desc, cast, Date
import logging


class K2Notifications(K2Obj):
    name = 'k2notifications'

    def __init__(self, *args, **kwargs):
        super().__init__()

    @classmethod
    def __generate_id(cls):
        '''Генерація ID'''
        generated_id = uuid.uuid4().hex[:32]
        return ''.join([generated_id[i:i + 4] for i in range(0, len(generated_id), 4)])

    @classmethod
    def send_notifications(cls, obj_id, user_list, url=None, description=None):
        pass

    @classmethod
    def save_approve_obj(cls,
                         obj_id,
                         object_row_id=None,
                         parentid='0',
                         user_list=[],
                         url=None,
                         description=None,
                         comment=''):
        from app import db
        from components.k2site.k2site.models import K2Approval, K2users, K2Usernotification
        if not obj_id or not url or not user_list:
            return jsonify({"error": "Object ID, URL or user list is missing"}), 400
        Session = sessionmaker(bind=cls.db)
        session = Session()
        try:
            # Проходимо по кожному користувачеві і створюємо або оновлюємо запис у таблиці K2Approval
            for user_id in user_list:
                # Перевіряємо чи існує запис з таким object_id і user_id
                existing_approval = session.query(K2Usernotification).filter_by(object_id=obj_id,
                                                                                user_id=user_id,
                                                                                parentid='0'
                                                                                ).first()
                notification_id = cls.__generate_id()  # Генеруємо унікальний notification_id
                if not existing_approval:
                # Якщо запис не існує, створюємо новий
                    new_approval = K2Usernotification(
                        notification_id=notification_id,
                        object_id=obj_id,
                        object_row_id=object_row_id,
                        user_id=user_id,
                        is_approved=-1,  # Спочатку статус не затверджено
                        description=description,
                        approval_url=url,
                        createuser=current_user.get_id(),
                        updateuser=current_user.get_id(),
                        createdate=datetime.now(),
                        updatedate=datetime.now(),
                        parentid=parentid,
                        active=1
                    )
                    session.add(new_approval)
                else:
                    existing_approval.is_approved = -1
                    existing_approval.updatedate = datetime.now()
                try:
                    session.commit()
                except Exception as commit_error:
                    session.rollback()
                    return jsonify({"error": str(commit_error)}), 500

            return jsonify({"status": "success", "message": "Approval requests created/updated"}), 200
        except Exception as e:
            return jsonify({"error": str(e)}), 500
        finally:
            session.close()

    @classmethod
    def save_approve_obj_return(cls,
                                notification_id,
                                obj_id,
                                object_row_id=None,
                                is_approved=1,
                                url=None,
                                description=None,
                                comment=''):
        from app import K2
        from components.k2site.k2site.models import K2Approval, K2users, K2Usernotification
        from app import socketio
        from sqlalchemy.orm import sessionmaker
        from datetime import datetime

        if not obj_id:
            return jsonify({"error": "Object ID, URL or user list is missing"}), 400

        Session = sessionmaker(bind=cls.db)
        session = Session()

        try:
            # Знайдемо запис батьківського документа (де parentid = 0)
            parent_record = session.query(K2Usernotification).filter_by(object_id=obj_id, parentid='0').first()

            if parent_record:
                # Перевірити, чи існує запис з ненульовим parentid
                existing_approval_record = (session.query(K2Usernotification)
                                            .filter(K2Usernotification.object_id == obj_id,
                                                    K2Usernotification.is_approved == '0',
                                                    K2Usernotification.user_id == parent_record.createuser,
                                                    K2Usernotification.parentid != '0')
                                            .first())

                if not existing_approval_record:
                    try:
                        # Створюємо новий запис із parentid, який дорівнює notification_id батьківського запису
                        new_approval_record = K2Usernotification(
                            notification_id=K2.generate_id(),
                            object_id=obj_id,
                            user_id=parent_record.createuser,
                            object_row_id=parent_record.object_row_id,
                            parentid=notification_id,  # Використовуємо notification_id як parentid для нового запису
                            is_approved=is_approved,
                            description=parent_record.description,
                            approval_url=parent_record.approval_url,
                            createdate=datetime.now(),
                            updatedate=datetime.now(),
                            createuser=parent_record.user_id,  # Використовуємо createuser батьківського запису
                            comment=comment
                        )
                        session.add(new_approval_record)
                    except Exception as e:
                        K2.logging_message(status=K2.log_error, message=f"{e}")
                else:
                    existing_approval_record.is_approved = is_approved
                    existing_approval_record.updatedate = datetime.now(),
                    existing_approval_record.updateuser = parent_record.user_id


                # Оновлюємо статус батьківського запису, якщо документ затверджений
                if is_approved == 1:
                    parent_record.is_approved = 1
                    parent_record.updatedate = datetime.now(),
                else:
                    parent_record.is_approved = 0
                    parent_record.comment = comment
                    parent_record.updatedate = datetime.now()

                session.commit()

                # Відправляємо відповідь про успіх на фронтенд
                return jsonify({"status": "success", "message": "Новий запис створено і статус оновлено!"}), 200

            else:
                # Якщо батьківський запис не знайдено
                return jsonify({"error": "Батьківський документ не знайдено."}), 400

        except Exception as e:
            # Якщо сталася помилка, скасовуємо транзакцію та фіксуємо помилку
            session.rollback()
            K2.logging_message(status=K2.log_error, message=f"{e}")
            return jsonify({"error": str(e)}), 500

        finally:
            session.close()


    @classmethod
    def send_approval_request(cls,  user_list, obj_id,  object_row_id=None, parentid='0', url=None, description=None, comment=None):
        """
        Функція для відправки запиту на затвердження документу
        Відправляє запит від імені користувача
        """
        from components.k2site.k2site.models import K2users
        from app import db
        try:
            # Спочатку зберігаємо запити на затвердження
            cls.save_approve_obj(obj_id, object_row_id, parentid, user_list, url, description, comment)
            # #Відправляємо сповіщення на email
            # subject = 'Погодження документа'
            # email_template = description
            #
            # # Отримуємо користувачів зі списку для погодження
            # from .k2mail import K2Mail
            # users = K2users.query.filter(K2users.user_id.in_(user_list), K2users.active == 1).all()
            # for user in users:
            #     if user.email:
            #         K2Mail().send_email(subject, user.email, email_template, url)

            return jsonify({"status": "success", "message": "Approval requests and emails sent"}), 200

        except Exception as e:
            db.session.rollback()  # Відкочуємо транзакцію в разі помилки
            return jsonify({"error": str(e)}), 500
        finally:
            db.session.close()


    @classmethod
    def send_system_request(cls):
        pass


    @classmethod
    def get_user_notifications(cls):
        """
        Отримує непідтверджені сповіщення для поточного користувача
        """
        notifications = []
        from components.k2site.k2site.models import K2users, K2Usernotification
        from app import db
        try:
            # Отримуємо ідентифікатор поточного користувача
            current_user_id = current_user.get_id()

            # Створюємо два аліаси для таблиці K2users
            creator_user = aliased(K2users)
            approver_user = aliased(K2users)

            # Виконуємо JOIN для отримання необхідних даних
            today = datetime.utcnow().date()

            unapproved_notifications = db.session.query(
                K2Usernotification.notification_id,
                K2Usernotification.object_id,
                K2Usernotification.object_row_id,
                K2Usernotification.is_approved,
                K2Usernotification.user_id,
                K2Usernotification.approval_url,
                K2Usernotification.description,
                K2Usernotification.createdate,
                K2Usernotification.createuser,
                K2Usernotification.updatedate,
                K2Usernotification.createuser,
                K2Usernotification.updateuser,
                K2Usernotification.parentid,
                K2Usernotification.comment,
                creator_user.name.label('creator_name'),
                creator_user.photo.label('creator_photo'),
                approver_user.name.label('approver_name'),
                approver_user.photo.label('approver_photo')
            ).join(
                creator_user, K2Usernotification.createuser == creator_user.user_id
            ).join(
                approver_user, K2Usernotification.user_id == approver_user.user_id
            ).filter(
                K2Usernotification.active == 1
            ).filter(
                or_(
                    K2Usernotification.is_approved == -1,  # Показуємо всі, якщо is_approved == -1
                    and_(
                        K2Usernotification.is_approved != -1,
                        # Інші статуси
                        #cast(K2Usernotification.updatedate, Date) == today  # Тільки за поточний день
                    )
                )
            ).filter(
                or_(
                    K2Usernotification.user_id == current_user_id,
                    K2Usernotification.createuser == current_user_id  # Друга умова
                )
            ).order_by(
                desc(K2Usernotification.updatedate)
            ).limit(5).all()


            # Формуємо список сповіщень у зручний для повернення формат
            notifications = [
                {
                    "notification_id": notif.notification_id,
                    "user_id": notif.user_id,
                    "object_id": notif.object_id,
                    "object_row_id": notif.object_row_id,
                    "is_approved": notif.is_approved,
                    "parentid": notif.parentid,
                    "approval_url": notif.approval_url,
                    "description": notif.description,
                    "createdate": notif.createdate.strftime('%Y-%m-%d %H:%M:%S'),
                    "updatedate":notif.updatedate.strftime('%Y-%m-%d %H:%M:%S'),
                    "createuser": notif.createuser,
                    "updateuser": notif.updateuser,
                    "creator_name": notif.creator_name,
                    "creator_photo": notif.creator_photo if notif.creator_photo else None ,
                    "approver_name": notif.approver_name,
                    "approver_photo": notif.approver_photo if notif.approver_photo else None,
                    "comment": notif.comment,
                }
                for notif in unapproved_notifications
            ]
            return  notifications

        except Exception as e:
            notifications = []
            logging.error(e)
            return notifications