import logging
import os
from datetime import datetime
import uuid
import re
import json

from flask_login import LoginManager, current_user
from flask import request, send_from_directory
from sqlalchemy.orm import sessionmaker

from k2.k2obj import K2Obj


class K2Settings(K2Obj):
    '''клас збереження та пошуку налаштувань  для користувача, складу, контрагента, проекту
    '''
    name = 'k2settings'
    def __init__(self, *args, **kwargs):
        super().__init__()

    #     self._data_path = None
    #
    # @property
    # def data_path(self):
    #     """Getter"""
    #     return self._data_path
    #
    # @data_path.setter
    # def name_yml(self, name: str):
    #     """Setter method"""
    #     self._data_path = name

    def save_settings(self, settings, object, device=None, is_for_all=False):
        '''
        Method to save settings for a user, storage, counterpart, or project.

        Args:
            settings (dict): A dictionary containing the settings to be saved. This will be converted to a JSON string.
            user_id (str): The ID of the user for whom the settings are being saved.
            object (str): The object for which the settings are specific (name of grid or form or other object).
            device (str, optional): The device for which the settings are specific, if any. Defaults to None.
            is_for_all (Boolean, optional): Save settings for current user or for all.

        Examples:
            K2().settings.save_settings(settings=json_dict,object='users')
            K2().settings.save_settings(settings=json_dict, object='k2counterparts', is_for_all=True)

        Returns:
            None
        '''
        Session = sessionmaker(bind=self.db.engine)
        session = Session()
        try:
            from k2.k2cfg import K2
            from components.k2site.k2site.models import K2settings
            if is_for_all:
                user_id = current_user.get_id()
                if (K2().secur.is_superadmin(current_user)
                        or K2().secur.is_superadmin_branch(current_user)
                        or K2().secur.is_superadmin_root(current_user)
                        or K2().secur.is_superadmin_cloud(current_user)):
                    roleid = self.__current_usr_role(current_user.get_id())
                    projid = K2().get_user_project_id()
                    counterpart_id = K2().get_user_counterparts_id()
                    storage_id = K2().get_user_storage_id()
                    # Check if settings already exist
                    existing_setting = session.query(K2settings).filter_by(
                        user_id=user_id,
                        object=object,
                        projid=projid,
                        counterpart_id=counterpart_id,
                        storage_id=storage_id
                    ).first()

                    if existing_setting:
                        # Update existing settings
                        existing_setting.settings = settings
                        existing_setting.device = device
                        existing_setting.updateuser = current_user.get_id()
                        existing_setting.updatedate = datetime.now()
                    else:
                        # Create new settings
                        new_setting = K2settings(
                            settingsid=K2.generate_id(),
                            roleid=roleid,
                            user_id=user_id,
                            device=device,
                            settings=settings,  # Convert the settings dictionary to a JSON string
                            object=object,
                            projid=projid,
                            counterpart_id=counterpart_id,
                            storage_id=storage_id,
                            createuser=current_user.get_id(),
                            createdate=datetime.now(),
                            updateuser=current_user.get_id(),
                            updatedate=datetime.now(),
                            active=1
                        )
                        session.add(new_setting)
                else:
                    K2.logging_message(status=K2.log_warning,  message="Permission error", show_message=True)
            else:
                user_id = current_user.get_id()
                existing_setting = session.query(K2settings).filter_by(
                    user_id=user_id,
                    object=object,
                ).first()
                if existing_setting:
                    # Update existing settings
                    existing_setting.settings = settings
                    existing_setting.device = device
                    existing_setting.updateuser = current_user.get_id()
                    existing_setting.updatedate = datetime.now()
                else:
                    # Create new settings
                    roleid = self.__current_usr_role(current_user.get_id())
                    new_setting = K2settings(
                        settingsid=K2.generate_id(),
                        roleid=roleid,
                        user_id=user_id,
                        device=device,
                        settings=settings,  # Convert the settings dictionary to a JSON string
                        object=object,
                        createuser=current_user.get_id(),
                        createdate=datetime.now(),
                        updateuser=current_user.get_id(),
                        updatedate=datetime.now(),
                        active=1
                    )
                    session.add(new_setting)
            session.commit()
            K2.logging_message(status=K2.log_success, message=f"Settings saved", show_message=True)
        except Exception as e:
            K2.logging_message(status=K2.log_error, message=f"Error in method save_settings {e}", show_message=True)
        finally:
            session.close()

    def get_settings(self, object, user_id=None):
        '''
        Method to get settings for a user according to the hierarchy of user, storage, counterpart, and project.

        Args:
            user_id (str): The ID of the user for whom the settings are being retrieved.
            object (str): The object for which the settings are specific (name of grid or form or other object).

        Returns:
            dict or None: The settings dictionary if found, otherwise None.
        '''
        Session = sessionmaker(bind=self.db.engine)
        session = Session()

        try:
            from components.k2site.k2site.models import K2settings, K2users
            user_id = current_user.get_id()
            user = session.query(K2users).get(user_id)
            if not user:
                return None

            # Check for user-specific settings
            user_setting = session.query(K2settings).filter_by(user_id=user_id, object=object).first()
            if user_setting:
                return user_setting.settings

            # Check for storage-specific settings
            storage_setting = session.query(K2settings).filter_by(storage_id=user.storage_id, object=object).first()
            if storage_setting:
                return storage_setting.settings

            # Check for counterpart-specific settings
            counterpart_setting = session.query(K2settings).filter_by(counterpart_id=user.counterpart_id,
                                                                      object=object).first()
            if counterpart_setting:
                return counterpart_setting.settings

            # Check for project-specific settings
            proj_setting = session.query(K2settings).filter_by(projid=user.projid, object=object).first()
            if proj_setting:
                return proj_setting.settings

            return None
        except Exception as e:
            logging.error(f"Error in method get_settings {e}")
            return None
        finally:
            session.close()
